/* channel_mode.h: The ircd channel mode header.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_channel_mode_h
#define INCLUDED_channel_mode_h
#include "config.h"
#include "ircd_defs.h"

#define MODEBUFLEN      200
#define MAXMODEPARAMS   4 /* maximum mode changes per line */

struct membership;
extern void set_channel_mode(struct Client *, struct Client *, struct Channel *,
  struct membership *, int, char **, char *);
extern void init_chcap_usage_counts(void);
extern void set_chcap_usage_counts(struct Client * serv_p);
extern void unset_chcap_usage_counts(struct Client * serv_p);

/* can_send results */
#define CAN_SEND_NO        0
#define CAN_SEND_NONOP     1
#define CAN_SEND_OPV       2

#define CHFL_PEON          0x0000 /* normal member */
#define CHFL_CHANOP        0x0001 /* channel operator */
#define CHFL_VOICE         0x0002 /* the power to speak */
#define CHFL_DEOPPED       0x0004 /* deopped by us, bounce modes */

#define CHFL_BAN           0x0010 /* channel ban */
#define CHFL_EXCEPTION     0x0020 /* ban exception */
#define CHFL_INVEX         0x0080 /* invite exempt */

#define ONLY_SERVERS       0x0100 /* send to servers only */
#define ALL_MEMBERS        CHFL_PEON /* send to all members */
#define ONLY_CHANOPS       CHFL_CHANOP
#define ONLY_CHANOPSVOICED (CHFL_CHANOP|CHFL_VOICE)

#define is_chanop(x)        ((x) && (x)->flags & CHFL_CHANOP)
#define is_voiced(x)        ((x) && (x)->flags & CHFL_VOICE)
#define is_chanop_voiced(x) ((x) && (x)->flags & (CHFL_CHANOP|CHFL_VOICE))
#define is_deop(x)          ((x) && (x)->flags & CHFL_DEOPPED)

/* channel modes */
#define MODE_PRIVATE    0x0008
#define MODE_SECRET     0x0010
#define MODE_MODERATED  0x0020
#define MODE_TOPICLIMIT 0x0040
#define MODE_INVITEONLY 0x0080
#define MODE_NOPRIVMSGS 0x0100
#define MODE_BAN        0x0400
#define MODE_EXCEPTION  0x0800
#define MODE_INVEX      0x2000
#define MODE_HIDEOPS    0x4000
#define MODE_NOCOLOR    0x8000
/* channel modes requiring parameters */
#define MODE_QUERY     0
#define MODE_ADD       1
#define MODE_DEL      -1

/* name invisible */
#define SecretChannel(x)   ((x) && ((x)->mode.mode & MODE_SECRET))
/* channel not shown but names are */
#define HiddenChannel(x)   ((x) && ((x)->mode.mode & MODE_PRIVATE))
#define PubChannel(x)      ((!x) || ((x)->mode.mode & (MODE_PRIVATE | \
                            MODE_SECRET)) == 0)
#define ParanoidChannel(x) ((x) && ((x)->mode.mode & (MODE_PRIVATE | \
                            MODE_INVITEONLY))== (MODE_PRIVATE | \
                            MODE_INVITEONLY))
struct ChModeChange {
  char letter;
  const char *arg;
  int dir;
  int caps;
  int nocaps;
  int mems;
  struct Client *client;
};
struct ChCapCombo {
  int count;
  int cap_yes;
  int cap_no;
};
#endif /* INCLUDED_channel_mode_h */
