/* config.h: compile-time configuration editor
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_config_h
#define INCLUDED_config_h
#include "setup.h"

/* File Descriptor limit
 *
 * These limits are ultra-low to guarantee the server will work properly
 * in as many environments as possible.  They can probably be increased
 * significantly, if you know what you are doing.
 *
 * Note that HARD_FDLIMIT_ is modified with the maxclients setting in
 * configure.  You should only touch this if you know what you are doing!!!
 *
 * If you are using select for the IO loop, you may also need to increase
 * the value of FD_SETSIZE by editting the Makefile.  However, using
 * --enable-kqueue, --enable-devpoll, or --enable-poll if at all possible,
 * is highly recommended.
 *
 * VMS should be able to use any suitable value here, other operating
 * systems may require kernel patches, configuration tweaks, or ulimit
 * adjustments in order to exceed certain limits (e.g. 1024, 4096 fds).
 */
#define HARD_FDLIMIT_    MAX_CLIENTS + 60 + 20

/* XXX - MAX_BUFFER is mostly ignored. */
/* Maximum number of connections to allow.
 * MAX_BUFFER  is the number of fds to reserve, e.g. for clients
 *             exempt from limit.
 *
 * 10 file descriptors are reserved for logging, DNS lookups, etc.,
 * so MAX_CLIENTS + MAX_BUFFER + 10 must not exceed HARD_FDLIMIT.
 * NOTE: MAX_CLIENTS is set with configure now
 */
#define MAX_BUFFER      60

#ifndef VMS
/* Directory paths and filenames for UNIX systems.
 * IRCD_PREFIX is set using ./configure --prefix. The defaults should be fine.
 * NOTE: IF YOU CHANGE THESE, DO NOT USE MAKE INSTALL, COPY THE FILES MANUALLY.
 *
 * VMS systems, see below.
 *
 * IRCD_PREFIX - prefix for all directories
 * DPATH       - root directory of installation
 * BINPATH     - directory for binary files
 * ETCPATH     - directory for configuration files
 * LOGPATH     - directory for logfiles
 * MODPATH     - directory for modules
 * AUTOMODPATH - directory for autoloaded modules
 */
#define DPATH       IRCD_PREFIX
#define BINPATH     IRCD_PREFIX "/bin/"
#define MODPATH     IRCD_PREFIX "/modules"
#define AUTOMODPATH IRCD_PREFIX "/modules/autoload"
#define ETCPATH     IRCD_PREFIX "/etc"
#define LOGPATH     IRCD_PREFIX "/logs"
#define UHPATH      IRCD_PREFIX "/help/users"
#define HPATH       IRCD_PREFIX "/help/opers"

#define SPATH       BINPATH "/ircd" /* ircd executable */
#define SLPATH      BINPATH "/servlink" /* servlink executable */
#define CPATH       ETCPATH "/ircd.conf" /* ircd.conf file */
#define KPATH       ETCPATH "/kline.conf" /* kline file */
#define DLPATH      ETCPATH "/dline.conf" /* dline file */
#define XPATH       ETCPATH "/xline.conf" /* xline file */
#define RESVPATH    ETCPATH "/resv.conf" /* resv file */
#define GPATH       LOGPATH "/gline.log" /* gline logfile */
#define RPATH       ETCPATH "/ircd.rsa" /* ircd rsa private keyfile */
#define MPATH       ETCPATH "/ircd.motd" /* MOTD file */
#define LPATH       LOGPATH "/ircd.log" /* ircd logfile */
#define PPATH       ETCPATH "/ircd.pid" /* pid file */
#define OPATH       ETCPATH "/oper.motd" /* oper MOTD file */
#define LIPATH      ETCPATH "/links.txt" /* cached links file */

#else /* !VMS */

/* *PATH - directory locations and filenames for VMS.
 *
 * DPATH       - root directory of installation
 * BINPATH     - directory for binary files
 * ETCPATH     - directory for configuration files
 * LOGPATH     - directory for logfiles
 */

/* Note: IRCD_PREFIX not needed in VMS -larne */
#define DPATH   "IRCD$BASEDIR:"
#define BINPATH "IRCD$BINDIR:"
#undef MODPATH
#undef AUTOMODPATH
#define ETCPATH "IRCD$CONFDIR:"
#define LOGPATH "IRCD$LOGDIR:"

#define SPATH    BINPATH "IRCD.EXE"
#define SLPATH   BINPATH "SERVLINK.EXE"
#define CPATH    ETCPATH "IRCD.CONF"
#define KPATH    ETCPATH "KLINE.CONF"
#define DLPATH   ETCPATH "DLINE.CONF"
#define XPATH    ETCPATH "XLINE.CONF"
#define RESVPATH ETCPATH "RESV.CONF"
#define GPATH    LOGPATH "GLINE.LOG"
#define RPATH    ETCPATH "IRCD.RSA"
#define MPATH    ETCPATH "IRCD.MOTD"
#define LPATH    LOGPATH "IRCD.LOG"
#define PPATH    ETCPATH "IRCD.PID"
#define HPATH    ETCPATH "OPERS.TXT"
#define UHPATH   ETCPATH "USERS.TXT"
#define OPATH    ETCPATH "OPER.MOTD"
#define LIPATH   ETCPATH "LINKS.TXT"

#endif /* !VMS */

/* IGNORE_BOGUS_TS
 * Ignore bogus timestamps from other servers. Can cause desync, but allows
 * chanops to resync with a valid TS. Enable network wide, or not at all.
 */
#undef IGNORE_BOGUS_TS

/* HIDE_SERVERS_IPS
 * If this is undefined, opers will be unable to see the real IP of servers, but
 * admins will still be shown the real IP. If this is defined, admins can't see
 * the real IP either.
 */
#undef HIDE_SERVERS_IPS

/* HIDE_SPOOF_IPS
 * If this is undefined, opers will be allowed to see the real IP of spoofed
 * users in /trace etc. If this is defined they will be shown a masked IP.
 */
#undef HIDE_SPOOF_IPS

/* If this is defined only TS5 servers may link to the network. If your network
 * has old servers (hyb5, hyb6.0, +CSr) or hybserv you should NOT define this.
 */
#define TS5_ONLY

/* USE_ASCII_CASEMAP
 * Under rfc1459, the characters {}|~ are the lowercase of []\^ so a nick of
 * [foo] is the same as {foo} and a channel #[] is the same as #{}. If this is
 * defined they will no longer be treated as lowercase of each other, so [foo]
 * and {foo} could be two separate people.
 * Note: This must be the same network-wide. Your locale must be set to "C."
 */
#undef USE_ASCII_CASEMAP

/* USE_LOGFILE - log errors and such to LPATH */
#define USE_LOGFILE

/* USE_SYSLOG
 * If you wish to have the server send 'vital' messages about server through
 * syslog, define USE_SYSLOG. If this is defined with FNAME_USERLOG, syslog() is
 * used instead of the above file. It is not recommended that this option be
 * used unless you have the system administrator's permission beforehand.
 */
#undef USE_SYSLOG

#ifdef USE_SYSLOG
/* if you use syslog, control what goes in the log */
#undef SYSLOG_KILL /* log all operator kills */
#undef SYSLOG_SQUIT /* log remote squits */
#undef SYSLOG_CONNECT /* log remote connect messages */
#undef SYSLOG_USERS /* log user information */
#undef SYSLOG_OPER /* log successful OPER or CHALLENGEs */

/* LOG_FACILITY - facility to use for syslog() */
#define LOG_FACILITY LOG_LOCAL4
#endif /* USE_SYSLOG */

/* number of messages we can receive before we disconnect the remote client */
#define CLIENT_FLOOD 20

/* this defines the length of the nickname history. when a user changes nickname
 * or signs off, their old nickname is added to the top of the list. A higher
 * length means more physical memory, the memory is rarely swapped to disk.
 */
#define NICKNAMEHISTORYLENGTH 1000

/* Often net breaks for a short time and it's useful to try to establishing the
 * same connection again faster than CONNECTFREQUENCY. Don't keep trying bad
 * connections, it must have been open for certain minimum time (HANGONGOODLINK)
 * and we give the net few seconds to notice the split. -ThemBones
 */
#define HANGONRETRYDELAY 60 /* Recommended: 30-60 secs */
#define HANGONGOODLINK 3600 /* Recommended: 30-60 mins */

/* Max time from the nickname change that still causes KILL automatically to
 * switch for the current nick of that user. (seconds)
 */
#define KILLCHASETIMELIMIT 90 /* Recommended: 90 */

/* If the OS has SOMAXCONN use that value, otherwise use the value here for
 * the listen() backlog. 5 for AIX/SunOS, 25 for others
 */
#define HYBRID_SOMAXCONN 25
/* END OF CONFIGURABLE OPTIONS */

/* Default pre-allocations */
#define CHANNEL_HEAP_SIZE 256
#define BAN_HEAP_SIZE     128
#define CLIENT_HEAP_SIZE  256
#define LCLIENT_HEAP_SIZE 128
#define LINEBUF_HEAP_SIZE 128
#define USER_HEAP_SIZE    128
#define DNODE_HEAP_SIZE   256
#define TOPIC_HEAP_SIZE   256
#define MEMBER_HEAP_SIZE  256

#undef DEBUGMODE /* slows down your server drastically, for debug only */
#include "defaults.h"
#endif /* INCLUDED_config_h */
