/* fileio.h: A header for some built in command handlers.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_handlers_h
#define INCLUDED_handlers_h

/* m_functions execute protocol messages on this server:
 * int m_func(struct Client* client_p, struct Client* source_p, int parc,
 *   char* parv[]);
 *
 *  client_p is always NON-NULL, pointing to a *LOCAL* client structure (with an
 *   open socket connected!). This identifies the physical socket where the
 *   message originated (or which caused the m_function to be executed--some
 *   m_functions call others)
 *
 *  source_p is the source of the message, defined by the prefix part of the
 *   message if present. If not or prefix not found, then source_p==client_p.
 *
 *  *Always* true (if 'parse' and others are working correct):
 *  1) source_p->from == client_p  (note: client_p->from == client_p)
 *  2) MyConnect(source_p) <=> source_p == client_p (e.g. source_p *cannot* be a
 *     local connection, unless it's actually client_p.
 *  3) parc => number of variable parameter strings (if 0, parv is NULL)
 *  4) parv => NULL terminated list of parameter pointers,
 *     parv[0], sender (prefix string) or empty string
 *     parv[1] - parv[parc-1], pointers to additional parameters
 *     parv[parc] == NULL
 *     NOTE: parv[0] - parv[parc-1] are all non-NULL pointers
 */

struct Client;
/* generic handlers */
extern void ms_error(struct Client *, struct Client *, int, char **);
extern void m_error(struct Client *, struct Client *, int, char **);
extern void m_ignore(struct Client *, struct Client *, int, char **);
extern void m_not_oper(struct Client *, struct Client *, int, char **);
extern void m_registered(struct Client *, struct Client *, int, char **);
extern void m_unregistered(struct Client *, struct Client *, int, char **);
extern void m_unsupported(struct Client *, struct Client *, int, char **);
#ifdef FL_DEBUG
extern void mo_hash(struct Client *, struct Client *, int, char **);
#endif

#endif /* INCLUDED_handlers_h */
