/* hook.h: A header for the hooks into parts of ircd.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef __HOOK_H_INCLUDED
#define __HOOK_H_INCLUDED

#include "tools.h"

typedef struct {
  char *name;
  int id;
  dlink_list hooks;
} hook;
/* don't define the arguments to hookfn, because they can vary between hooks */
typedef int (*hookfn) (void *data);
/* used when a hook is called by an m_function */
struct hook_mfunc_data {
  struct Client *client_p;
  struct Client *source_p;
  int parc;
  char **parv;
};
struct hook_stats_data {
  struct Client *source_p;
  char statchar;
  const char *name;
};
struct hook_links_data {
  struct Client *client_p;
  struct Client *source_p;
  const char *mask;
};
struct hook_io_data {
  struct Client *connection;
   const char *data;
  unsigned int len;
};
struct hook_burst_channel {
  struct Client *client;
  struct Channel *chptr;
};

int hook_add_event(const char *, int *);
int hook_add_hook(const char *, hookfn);
int hook_call_event(int id, void *);
int hook_del_event(const char *);
int hook_del_hook(const char *event, hookfn);
void init_hooks(void);

#ifndef NDEBUG
extern int h_iosend_id;
extern int h_iorecv_id;
extern int h_iorecvctrl_id;
#endif
extern int h_burst_channel_id;
extern int h_doing_admin_id;
extern int h_doing_info_id;
extern int h_doing_links_id;
extern int h_doing_motd_id;
extern int h_doing_stats_id;
extern int h_doing_trace_id;
#endif
