/* ircd.h: A header for the ircd startup routines.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_ircd_h
#define INCLUDED_ircd_h

#include "config.h"
#include "tools.h"
#include "memory.h"

struct Client;
struct dlink_list;

struct SetOptions {
  int maxclients; /* max clients allowed */
  int maxauth; /* max clients in auth */
  int autoconn; /* autoconn enabled for all servers? */

  int idletime;

  int floodcount; /* Number of messages in 1 second */
  int ident_timeout; /* timeout for identd lookups */

  int spam_num;
  int spam_time;
  int split_delay;
  char operstring[REALLEN];
  char adminstring[REALLEN];
};

struct Counter {
  int server;   /* servers */
  int myserver; /* my servers */
  int oper;     /* Opers */
  int chan;     /* Channels */
  int local;    /* Local Clients */
  int total;    /* total clients */
  int invisi;   /* invisible clients */
  int unknown;  /* unknown connections */
  int max_loc;  /* MAX local clients */
  int max_tot;  /* MAX global clients */
  unsigned long totalrestartcount; /* Total client count ever */
};

extern struct SetOptions GlobalSetOptions; /* defined in ircd.c */

struct ServerState_t {
  int foreground;
};
extern struct ServerState_t server_state;

extern const char *debugmode;
extern int debuglevel;
extern int debugtty;
extern char *creation;
extern char *generation;
extern char *platform;
extern char *infotext[];
extern char *serno;
extern char *ircd_version;
extern const char *logFileName;
extern const char *pidFileName;
extern const char serveropts[];
extern int cold_start;
extern int dorehash;
extern int doremotd;
extern int kline_queued;
extern struct Client me;
extern dlink_list global_client_list;
extern struct Client *local[];
extern struct Counter Count;
#if 0
extern time_t CurrentTime;
#endif
extern struct timeval SystemTime;
#define CurrentTime SystemTime.tv_sec
extern time_t nextconnect;
extern int default_server_capabs;

extern int splitmode;
extern int splitchecking;
extern int split_users;
extern int split_servers;

extern dlink_list unknown_list;
extern dlink_list lclient_list;
extern dlink_list serv_list;
extern dlink_list global_serv_list;
extern dlink_list oper_list;
extern dlink_list dead_list;
extern dlink_list abort_list;

extern unsigned long get_maxrss(void);
extern void set_time(void);

extern int testing_conf;
#endif
