/* ircd_defs.h: ircd global definitions
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_ircd_defs_h
#define INCLUDED_ircd_defs_h

#include "config.h"

#define NICKLEN           16 /* 15+1 for \0 */
#define TOPICLEN          300
#define KICKLEN           100
#define KILLLEN           150
#define REALLEN           50
#define CHANNELLEN        50
#define REASONLEN         150
#define KEYLEN            24 /* 23+1 for \0 */
#define HOSTLEN           63 /* Length of hostname. Complies with RFC 1123 */
#define USERLEN           10
#define USERHOST_REPLYLEN (NICKLEN+USERLEN+HOSTLEN+5)
#define HELPLEN           80
#define OPERNICKLEN       NICKLEN+5 /* Length of operid's */

#define BUFSIZE           512 /* WARNING: *DONT* CHANGE THIS */
#define MAXRECIPIENTS     20
#define MAXBANLENGTH      1024

#define MAX_DATE_STRING   32 /* maximum string length for a date string */

/* For those unfamiliar with GNU format attributes, a is the 1 based
 * argument number of the format string, and b is the 1 based argument
 * number of the variadic ... */
#ifdef __GNUC__
#define AFP(a, b) __attribute__((format (printf, a, b)))
#else
#define AFP(a, b)
#endif

#include "s_log.h"
#include "send.h"

#ifdef SOFT_ASSERT
#ifdef __GNUC__
#define s_assert(expr) \
  do { \
    if (!(expr)) { \
      ilog(L_ERROR, "file: %s line: %d (%s): Assertion failed: (%s)", \
        __FILE__, __LINE__, __PRETTY_FUNCTION__, #expr); \
      sendto_realops_flags(UMODE_ALL, L_ALL, "file: %s line: %d (%s): " \
        "Assertion failed: (%s)", __FILE__, __LINE__, __PRETTY_FUNCTION__, \
        #expr); \
    } \
  } while(0)
#else
#define s_assert(expr) \
  do { \
    if (!(expr)) { \
      ilog(L_ERROR, "file: %s line: %d: Assertion failed: (%s)" __FILE__, \
        __LINE__, #expr); \
      sendto_realops_flags(UMODE_ALL, L_ALL, "file: %s line: %d: Assertion " \
        "failed: (%s)", __FILE__, __LINE__, #expr); \
    } \
  } while(0)
#endif
#else
#define s_assert(expr) assert(expr)
#endif

/* message return values */
#define CLIENT_EXITED    -2
#define CLIENT_PARSE_ERROR -1
#define CLIENT_OK        1

struct irc_inaddr {
  union {
    struct in_addr sin;
#ifdef IPV6
    struct in6_addr sin6;
#endif
  } sins;
};

struct irc_sockaddr {
  union {
    struct sockaddr_in sin;
#ifdef IPV6
    struct sockaddr_in6 sin6;
#endif
  } sins;
};

#ifdef IPV6
#define copy_s_addr(a, b) \
  do { \
    ((uint32_t *)a)[0] = ((uint32_t *)b)[0]; \
    ((uint32_t *)a)[1] = ((uint32_t *)b)[1]; \
    ((uint32_t *)a)[2] = ((uint32_t *)b)[2]; \
    ((uint32_t *)a)[3] = ((uint32_t *)b)[3]; \
  } while(0)

/* irc_sockaddr macros */
#define PS_ADDR(x) x->sins.sin6.sin6_addr.s6_addr /* s6_addr for pointer */
#define S_ADDR(x) x.sins.sin6.sin6_addr.s6_addr /* s6_addr for non pointer */
#define S_PORT(x) x.sins.sin6.sin6_port /* s6_port */
#define S_FAM(x) x.sins.sin6.sin6_family /* sin6_family */
#define SOCKADDR(x) x.sins.sin6 /* struct sockaddr_in6 for nonpointer */
#define PSOCKADDR(x) x->sins.sin6 /* struct sockaddr_in6 for pointer */

/* irc_inaddr macros */
#define IN_ADDR(x) x.sins.sin6.s6_addr
#define IPV4_MAPPED(x) ((uint32_t *)x.sins.sin6.s6_addr)[3]
#define PIN_ADDR(x) x->sins.sin6.s6_addr /* For Pointers */
#define IN_ADDR2(x) x.sins.sin6

#define DEF_FAM AF_INET6

#else
#define copy_s_addr(a, b) a = b

#define PS_ADDR(x)   x->sins.sin.sin_addr.s_addr /* s_addr for pointer */
#define S_ADDR(x)    x.sins.sin.sin_addr.s_addr /* s_addr for nonpointer */
#define S_PORT(x)    x.sins.sin.sin_port /* sin_port */
#define S_FAM(x)     x.sins.sin.sin_family /* sin_family */
#define SOCKADDR(x)  x.sins.sin /* struct sockaddr_in */
#define PSOCKADDR(x) x->sins.sin /* struct sockaddr_in */

#define PIN_ADDR(x)  x->sins.sin.s_addr
#define IN_ADDR(x)   x.sins.sin.s_addr

#ifndef AF_INET6
#define AF_INET6 10 /* Dummy AF_INET6 declaration */
#endif

#define DEF_FAM AF_INET
#endif
#endif /* INCLUDED_ircd_defs_h */
