/* m_info.h: A header for the information sent by /info
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_m_info_h
#define INCLUDED_m_info_h

#include "config.h"

typedef struct Information {
  const char *name; /* name of item */
  const char *strvalue; /* value of item if it's a string */
  int intvalue; /* value of item if it's an integer */
  const char *desc; /* short description of item */
} Info;

Info MyInformation[] = {
  {
    "HARD_FDLIMIT",
    "",
    HARD_FDLIMIT_,
    "Maximum file descriptors available",
  },
  {
    "MAX_CLIENTS",
    "",
    MAX_CLIENTS,
    "Default maximum clients",
  },
  {
    "EINTERFACE",
    SELECT_TYPE,
    0,
    "Method of multiplexed I/O",
  },
  {
    "KILLCHASETIME",
    "",
    KILLCHASETIMELIMIT,
    "Nickname tracing time limit",
  },
  {
    "SOMAXCONN",
    "",
#ifdef SOMAXCONN
    SOMAXCONN,
#else
    HYBRID_SOMAXCONN,
#endif
    "Maximum queue length of pending connections",
  },
  {
    "CLIENT_FLOOD",
#ifdef CLIENT_FLOOD
    "",
    CLIENT_FLOOD,
#else
    "Off",
    0,
#endif
    "Client excess flood threshold",
  },
  {
    "CMDLINE_CONFIG",
#ifdef CMDLINE_CONFIG
    "On",
#else
    "Off",
#endif
    0,
    "Allow command-line specification of configuration file",
  },
  {
    "DEBUGMODE",
#ifdef DEBUGMODE
    "Yes",
#else
    "No",
#endif
    0,
    "Development and testing mode",
  },
  {
    "IPV6",
#ifdef IPV6
    "Yes",
#else
    "No",
#endif
    0,
    "Internet Protocol version 6 support",
  },
  {
    "JOIN_PART_TTL",
    "",
    JOIN_LEAVE_COUNT_EXPIRE_TIME,
    "Join/part tracking time-to-live",
  },
  {
    "JOIN_PART_COUNT",
    "",
    MAX_JOIN_LEAVE_COUNT,
    "Join/part threshold for spam bot detection",
  },
  {
    "JOIN_PART_INTERVAL",
    "",
    MIN_JOIN_LEAVE_TIME,
    "Join/part interval for spam bot detection",
  },
  {
    "JOIN_PART_PACE",
    "",
    OPER_SPAM_COUNTDOWN,
    "Join/part notification interval",
  },
  {
    "CPATH",
#ifdef CPATH
    CPATH,
#else
    "None",
#endif
    0,
    "Path to default configuration file",
  },
  {
    "DPATH",
#ifdef DPATH
    DPATH,
#else
    "None",
#endif
    0,
    "Directory containing configuration files",
  },
  {
    "DLPATH",
#ifdef DLPATH
    DLPATH,
#else
    "None",
#endif
    0,
    "Path to D-Line file",
  },
  {
    "RESVPATH",
#ifdef RESVPATH
    RESVPATH,
#else
    "None",
#endif
    0,
    "Path to reservation file",
  },
  {
    "KPATH",
#ifdef KPATH
    KPATH,
#else
    "None",
#endif
    0,
    "Path to K-Line file",
  },
  {
    "LPATH",
#ifdef LPATH
    LPATH,
#else
    "None",
#endif
    0,
    "Path to log file",
  },
  {
    "HPATH",
#ifdef HPATH
    HPATH,
#else
    "None",
#endif
    0,
    "Path to operator help files",
  },
  {
    "UHPATH",
#ifdef UHPATH
    UHPATH,
#else
    "None",
#endif
    0,
    "Path to user help files",
  },
  {
    "MPATH",
#ifdef MPATH
    MPATH,
#else
    "None",
#endif
    0,
    "Path to message of the day file",
  },
  {
    "OPATH",
#ifdef OPATH
    OPATH,
#else
    "None",
#endif
    0,
    "Path to operator message of the day file",
  },
  {
    "PPATH",
#ifdef PPATH
    PPATH,
#else
    "None",
#endif
    0,
    "Path to process identification file",
  },
  {
    "SPATH",
#ifdef SPATH
    SPATH,
#else
    "None",
#endif
    0,
    "Path to server executable",
  },
  {
    "USERHISTORY",
    "",
    NICKNAMEHISTORYLENGTH,
    "Size of WHOWAS tracking array",
  },
  {
    "MAX_BUFFER",
    "",
    MAX_BUFFER,
    "Maximum buffer connections",
  },
  {
    "SYSLOG",
#ifdef USE_SYSLOG
    "Yes",
#else
    "No",
#endif
    0,
    "Log errors to system event log",
  },
  {
    "PACE_CONNECT",
#ifdef PACE_CONNECT
    "Yes",
#else
    "No",
#endif
    0,
    "Connection pacing",
  },
  {
    "SSL",
#ifdef HAVE_LIBCRYPTO
    "Yes",
#else
    "No",
#endif
    0,
    "OpenSSL cryptography support",
  },
  {
    "ZLIB",
#ifdef HAVE_LIBZ
    "Yes",
#else
    "No",
#endif
    0,
    "Zip compression support",
  },

  /* since we don't want to include the world here, NULL probably isn't defined
   * by the time we read this, just use plain 0 instead. 0 is guaranteed by the
   * language to be assignable to ALL built in types with the correct results.
   */
  {0, 0, 0, 0}
};

#endif /* INCLUDED_m_info_h */
