/* memory.h: A header for the memory functions.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef _I_MEMORY_H
#define _I_MEMORY_H

#include "ircd_defs.h"
#include "setup.h"
#include "balloc.h"

/* Needed to use uintptr_t for some pointer manipulation. */
#ifdef __VMS
#include inttypes
#else /* Not VMS */
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#else /* No inttypes.h */
#ifndef HAVE_UINTPTR_T
typedef unsigned long uintptr_t;
#endif
#endif
#endif

extern void outofmemory(void);

extern void *MyMalloc(size_t size);
extern void *MyRealloc(void *x, size_t y);
extern void MyFree(void *x);
extern void _DupString(char **x, const char *y);
/* forte/darwin dont like double declarations, only use inlines with GNUC */

#ifndef __APPLE__
#ifdef __GNUC__
extern inline void * MyMalloc(size_t size)
{
  void *ret = calloc(1, size);
  if (ret == NULL)
    outofmemory();
  return ret;
}
extern inline void * MyRealloc(void *x, size_t y)
{
  void *ret = realloc(x, y);
  if (ret == NULL)
    outofmemory();
  return ret;
}
extern inline void MyFree(void *x)
{
  if (x != NULL)
    free(x);
}
extern inline void _DupString(char **x, const char *y)
{
  (*x) = malloc(strlen(y) + 1);
  if (x == NULL)
    outofmemory();
  strcpy((*x), y);
}
#endif /* __GNUC__ */
#endif /* __APPLE__ */

#define DupString(x,y) _DupString(&x, y)

#ifndef WE_ARE_MEMORY_C
#undef strdup
#undef malloc
#undef realloc
#undef calloc
#undef free
#define malloc do_not_call_old_memory_functions!call_My*functions
#define calloc do_not_call_old_memory_functions!call_My*functions
#define realloc do_not_call_old_memory_functions!call_My*functions
#define strdup do_not_call_old_memory_functions!call_My*functions
#define free do_not_call_old_memory_functions!call_My*functions
#endif

#endif /* _I_MEMORY_H */
