/* modules.h: A header for the modules functions.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_modules_h
#define INCLUDED_modules_h
#include "config.h"
#include "setup.h"
#include "parse.h"
#ifdef HAVE_SHL_LOAD
#include <dl.h>
#endif
#if !defined(STATIC_MODULES) && defined(HAVE_DLFCN_H)
#include <dlfcn.h>
#endif
#include "ircd_handler.h"
#include "msg.h"
#include "memory.h"
#define HAVE_ENCAP_HANDLERS

#ifndef STATIC_MODULES
struct module {
  char *name;
  char *version;
  void *address;
  int core;
  void (*modremove) (void);
};

struct module_path {
  char path[MAXPATHLEN];
};
/* add a path */
void mod_add_path(const char *path);
void mod_clear_paths(void);
/* load a module */
extern void load_module(char *path);
/* load all modules */
extern void load_all_modules(int warn);
/* load core modules */
extern void load_core_modules(int);

extern void _modinit(void);
extern void _moddeinit(void);

extern int unload_one_module(char *, int);
extern int load_one_module(char *, int);
extern int load_a_module(char *, int, int);
extern int findmodule_byname(char *);
extern char *irc_basename(char *);
extern void modules_init(void);
#else /* STATIC_MODULES */

/* Core modules */
extern struct Message die_msgtab;
extern struct Message kick_msgtab;
extern struct Message kill_msgtab;
extern struct Message privmsg_msgtab;
extern struct Message notice_msgtab;
extern struct Message mode_msgtab;
extern struct Message nick_msgtab;
extern struct Message client_msgtab;
extern struct Message part_msgtab;
extern struct Message quit_msgtab;
extern struct Message server_msgtab;
extern struct Message sjoin_msgtab;
extern struct Message squit_msgtab;

/* Normal modules */
extern struct Message accept_msgtab;
extern struct Message admin_msgtab;
extern struct Message away_msgtab;
extern struct Message capab_msgtab;
#ifdef HAVE_LIBCRYPTO
extern struct Message challenge_msgtab;
extern struct Message cryptlink_msgtab;
#endif
extern struct Message close_msgtab;
extern struct Message connect_msgtab;
extern struct Message encap_msgtab;
extern struct Message etrace_msgtab;
extern struct Message ojoin_msgtab;
extern struct Message forcejoin_msgtab;
extern struct Message forcepart_msgtab;
extern struct Message gline_msgtab;
extern struct Message help_msgtab;
extern struct Message uhelp_msgtab;
extern struct Message info_msgtab;
extern struct Message invite_msgtab;
extern struct Message ison_msgtab;
extern struct Message join_msgtab;
extern struct Message kline_msgtab;
extern struct Message dline_msgtab;
extern struct Message knock_msgtab;
extern struct Message links_msgtab;
extern struct Message list_msgtab;
extern struct Message locops_msgtab;
extern struct Message lusers_msgtab;
extern struct Message map_msgtab;
extern struct Message motd_msgtab;
extern struct Message omotd_msgtab;
extern struct Message names_msgtab;
extern struct Message olist_msgtab;
extern struct Message oper_msgtab;
extern struct Message operwall_msgtab;
extern struct Message opme_msgtab;
extern struct Message pass_msgtab;
extern struct Message ping_msgtab;
extern struct Message pong_msgtab;
extern struct Message post_msgtab;
extern struct Message get_msgtab;
extern struct Message put_msgtab;
extern struct Message rehash_msgtab;
extern struct Message restart_msgtab;
extern struct Message resv_msgtab;
extern struct Message unresv_msgtab;
extern struct Message nickserv_msgtab;
extern struct Message ns_msgtab;
extern struct Message chanserv_msgtab;
extern struct Message cs_msgtab;
extern struct Message memoserv_msgtab;
extern struct Message ms_msgtab;
extern struct Message operserv_msgtab;
extern struct Message os_msgtab;
extern struct Message identify_msgtab;
extern struct Message set_msgtab;
extern struct Message stats_msgtab;
extern struct Message svinfo_msgtab;
extern struct Message takeover_msgtab;
extern struct Message tb_msgtab;
extern struct Message testline_msgtab;
extern struct Message time_msgtab;
extern struct Message topic_msgtab;
extern struct Message trace_msgtab;
extern struct Message kline_msgtab;
extern struct Message dline_msgtab;
extern struct Message gline_msgtab;
extern struct Message user_msgtab;
extern struct Message userhost_msgtab;
extern struct Message users_msgtab;
extern struct Message version_msgtab;
extern struct Message wallops_msgtab;
extern struct Message who_msgtab;
extern struct Message whois_msgtab;
extern struct Message whowas_msgtab;
extern struct Message xline_msgtab;
extern struct Message unxline_msgtab;
extern void load_all_modules(int check);
#endif /* STATIC_MODULES */
#endif /* INCLUDED_modules_h */
