/* msg.h: A header for the message handler structure.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#ifndef INCLUDED_msg_h
#define INCLUDED_msg_h

#include "config.h"
#include "ircd_handler.h"

struct Client;
/* Message table structure */
struct Message {
  const char *cmd;
  unsigned int count; /* number of times command used */
  unsigned int rcount; /* number of times command used by server */
  unsigned int parameters; /* required parameters */
  unsigned int maxpara; /* maximum permitted parameters */
  unsigned int flags; /* bit 0 set means only one use ~2 seconds -SRB */
  unsigned long bytes; /* bytes received for this message */
  /* client_p = Connected client ptr
   * source_p = Source client ptr
   * parc = parameter count
   * parv = parameter variable array
   */
  /* handlers: UNREGISTERED, CLIENT, SERVER, OPER, LAST */
  MessageHandler handlers[LAST_HANDLER_TYPE];
};
/* Constants */
#define MFLG_SLOW   0x01 /* Command can be executed once per ~2 seconds */
#define MFLG_UNREG  0x02 /* Command available to unregistered clients */
#define MFLG_IGNORE 0x04 /* drop if from unregistered client */
#define MAXPARA     15

#endif /* INCLUDED_msg_h */
