/* m_mode.c: Sets a user or channel mode.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "client.h"
#include "hash.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_user.h"
#include "s_conf.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "packet.h"

static void m_mode(struct Client *, struct Client *, int, char **);

struct Message mode_msgtab = {
  "MODE", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_mode, m_mode, m_ignore, m_mode}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&mode_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&mode_msgtab);
}

const char *_version = "1.73.2.2";
#endif

/* m_mode - MODE command handler
 *  parv[0] = sender prefix
 *  parv[1] = channel
 */
static void m_mode(struct Client * client_p, struct Client * source_p, int parc,
  char *parv[])
{
  struct Channel *chptr = NULL;
  struct membership *msptr;
  static char modebuf[MODEBUFLEN];
  static char parabuf[MODEBUFLEN];
  int n = 2;
  if (EmptyString(parv[1])) {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0],
      "MODE");
    return;
  }
  /* Now, try to find the channel in question */
  if (!IsChanPrefix(parv[1][0])) {
    /* if here, it has to be a non-channel name */
    user_mode(client_p, source_p, parc, parv);
    return;
  }
  if (!check_channel_name(parv[1])) {
    sendto_one(source_p, form_str(ERR_BADCHANNAME), me.name, parv[0],
      (unsigned char *)parv[1]);
    return;
  }
  chptr = hash_find_channel(parv[1]);
  if (chptr == NULL) {
    sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL), me.name, parv[0],
      parv[1]);
    return;
  }
  /* Now know the channel exists */
  if (parc < n + 1) {
    channel_modes(chptr, source_p, modebuf, parabuf);
    sendto_one(source_p, form_str(RPL_CHANNELMODEIS), me.name, parv[0], parv[1],
      modebuf, parabuf);
    sendto_one(source_p, form_str(RPL_CREATIONTIME), me.name, parv[0], parv[1],
      chptr->channelts);
  }
  else if (IsServer(source_p))
    set_channel_mode(client_p, source_p, chptr, NULL, parc - n, parv + n,
      chptr->chname);
  else {
    msptr = find_channel_membership(chptr, source_p);
    if (is_deop(msptr))
      return;
    /* Finish the flood grace period... */
    if (MyClient(source_p) && !IsFloodDone(source_p)) {
      if ((parc == 3) && (parv[2][0] == 'b') && (parv[2][1] == '\0'));
      else
        flood_endgrace(source_p);
    }
    set_channel_mode(client_p, source_p, chptr, msptr, parc - n, parv + n,
      chptr->chname);
  }
}
