/* m_admin.c: Sends administrative information to a user.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_conf.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "hook.h"
#include "modules.h"

static void m_admin(struct Client *, struct Client *, int, char **);
static void mr_admin(struct Client *, struct Client *, int, char **);
static void ms_admin(struct Client *, struct Client *, int, char **);
static void do_admin(struct Client * source_p);

struct Message admin_msgtab = {
  "ADMIN", 0, 0, 0, 0, MFLG_SLOW | MFLG_UNREG, 0,
  {mr_admin, m_admin, ms_admin, m_ignore, ms_admin}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&admin_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&admin_msgtab);
}
const char *_version = "1.50.2.4";
#endif

/* mr_admin - ADMIN command handler
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void mr_admin(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  static time_t last_used = 0L;
  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime) {
    sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, BadPtr(parv[0]) ? "*" :
      parv[0], "ADMIN");
    return;
  }
  else
    last_used = CurrentTime;
  do_admin(source_p);
}
/* m_admin - ADMIN command handler
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void m_admin(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  static time_t last_used = 0L;
  if (parc > 1) {
    if (hunt_server(client_p, source_p, ":%s ADMIN :%s", 1, parc, parv) !=
        HUNTED_ISME)
      return;
    if ((last_used + ConfigFileEntry.pace_wait_simple) > CurrentTime) {
      sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, parv[0], "ADMIN");
      return;
    }
    else
      last_used = CurrentTime;
    if (!ConfigServerHide.disable_remote) {
      if (hunt_server(client_p, source_p, ":%s ADMIN :%s", 1, parc, parv) !=
          HUNTED_ISME)
        return;
    }
  }
  do_admin(source_p);
}

/* ms_admin - ADMIN command handler, used for OPERS as well
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void ms_admin(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (hunt_server(client_p, source_p, ":%s ADMIN :%s", 1, parc, parv) !=
      HUNTED_ISME)
    return;
  if (IsClient(source_p))
    do_admin(source_p);
}

/* do_admin
 * inputs       - pointer to client to report to
 * side effects - admin info is sent to client given
 */
static void do_admin(struct Client * source_p)
{
  const char *nick;
  if (IsPerson(source_p))
    hook_call_event(h_doing_admin_id, source_p);
  nick = BadPtr(source_p->name) ? "*" : source_p->name;
  sendto_one(source_p, form_str(RPL_ADMINME), me.name, nick, me.name);
  if (AdminInfo.name != NULL)
    sendto_one(source_p, form_str(RPL_ADMINLOC1), me.name, nick,
      AdminInfo.name);
  if (AdminInfo.description != NULL)
    sendto_one(source_p, form_str(RPL_ADMINLOC2), me.name, nick,
      AdminInfo.description);
  if (AdminInfo.email != NULL)
    sendto_one(source_p, form_str(RPL_ADMINEMAIL), me.name, nick,
      AdminInfo.email);
}
