/* m_away.c: Sets/removes away status on a user.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_conf.h"
#include "s_serv.h"
#include "packet.h"

static void m_away(struct Client *, struct Client *, int, char **);

struct Message away_msgtab = {
  "AWAY", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_away, m_away, m_ignore, m_away}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&away_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&away_msgtab);
}
const char *_version = "1.47.2.3";
#endif

/* m_away
 *  parv[0] = sender prefix
 *  parv[1] = away message
 */
static void m_away(struct Client * client_p, struct Client * source_p, int parc,
  char *parv[])
{
  char *away, *awy2 = parv[1];
  if (MyClient(source_p) && !IsFloodDone(source_p))
    flood_endgrace(source_p);
  if (!IsClient(source_p))
    return;
  away = source_p->user->away;
  if (parc < 2 || !*awy2) {
    /* Marking as not away */
    if (away) {
      /* we now send this only if they were away before -is */
      sendto_server(client_p, NULL, NOCAPS, NOCAPS, ":%s AWAY", parv[0]);
      MyFree(away);
      source_p->user->away = NULL;
    }
    if (MyConnect(source_p))
      sendto_one(source_p, form_str(RPL_UNAWAY), me.name, parv[0]);
    return;
  }
  /* Marking as away */
  if (MyConnect(source_p) && !IsOper(source_p) && (CurrentTime -
      source_p->user->last_away) < ConfigFileEntry.pace_wait) {
    sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, parv[0], "AWAY");
    return;
  }
  source_p->user->last_away = CurrentTime;
  if (strlen(awy2) > (size_t) TOPICLEN)
    awy2[TOPICLEN] = '\0';
  /* we now send this only if they weren't away already -is */
  if (!away)
    sendto_server(client_p, NULL, NOCAPS, NOCAPS, ":%s AWAY :%s", parv[0],
      awy2);
  else
    MyFree(away);
  away = (char *)MyMalloc(strlen(awy2) + 1);
  strcpy(away, awy2);
  source_p->user->away = away;
  if (MyConnect(source_p))
    sendto_one(source_p, form_str(RPL_NOWAWAY), me.name, parv[0]);
}
