/* m_close.c: Closes all unregistered connections.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void mo_close(struct Client *, struct Client *, int, char **);

struct Message close_msgtab = {
  "CLOSE", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_close}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&close_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&close_msgtab);
}
const char *_version = "1.36.2.2";
#endif

/* mo_close - CLOSE message handler
 * by Darren Reed Jul 13 1992.
 */
static void mo_close(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  struct Client *target_p;
  dlink_node *ptr;
  dlink_node *ptr_next;
  int closed = 0;

  DLINK_FOREACH_SAFE(ptr, ptr_next, unknown_list.head) {
    target_p = ptr->data;
    sendto_one(source_p, form_str(RPL_CLOSING), me.name, parv[0],
      target_p->localClient->sockhost, target_p->status);
    (void)exit_client(target_p, target_p, target_p, "Unknown connection");
    closed++;
  }
  sendto_one(source_p, form_str(RPL_CLOSEEND), me.name, parv[0], closed);
}
