/* m_ison.c: Provides a single line answer of whether a user is online.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "irc_string.h"
#include "sprintf_irc.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_conf.h"
#include "s_serv.h"
#include "hash.h"

static void m_ison(struct Client *, struct Client *, int, char **);

struct Message ison_msgtab = {
  "ISON", 0, 0, 1, 1, MFLG_SLOW, 0,
  {m_unregistered, m_ison, m_ignore, m_ignore, m_ison}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&ison_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&ison_msgtab);
}
const char *_version = "1.44.2.2";
#endif

static char buf[BUFSIZE];
static char buf2[BUFSIZE];

/* m_ison added by Darren Reed 13/8/91 to act as an efficent user indicator
 * with respect to cpu/bandwidth used. Implemented for NOTIFY feature in
 * clients. Designed to reduce number of whois requests. Can process
 * nicknames in batches as long as the maximum buffer length.
 */
static void m_ison(struct Client * client_p, struct Client * source_p, int parc, char *parv[])
{
  struct Client *target_p;
  char *nick;
  char *p;
  char *current_insert_point, *current_insert_point2;
  int len;
  int i;
  int done = 0;
  current_insert_point2 = buf2;
  *buf2 = '\0';
  ircsprintf(buf, form_str(RPL_ISON), me.name, parv[0]);
  len = strlen(buf);
  current_insert_point = buf + len;
  /*
   * rfc1489 is ambigious about how to handle ISON this should handle both
   * interpretations.
   */
  for (i = 1; i < parc; i++) {
    for (nick = strtoken(&p, parv[i], " "); nick; nick = strtoken(&p, NULL,
         " ")) {
      target_p = find_person(nick);
      if (target_p == NULL && (IsOper(source_p) ||
          !ConfigServerHide.hide_servers)) {
        target_p = find_server(nick);
      }
      if (target_p != NULL) {
        len = strlen(target_p->name);
        if ((current_insert_point + (len + 5)) < (buf + sizeof(buf))) {
          memcpy((void *)current_insert_point,
            (void *)target_p->name, len);
          current_insert_point += len;
          *current_insert_point++ = ' ';
        }
        else {
          done = 1;
          break;
        }
      }
    }
    if (done)
      break;
  }
  /*
   * current_insert_point--; Do NOT take out the trailing space, it breaks
   * ircII --Rodder
   */
  *current_insert_point = '\0';
  *current_insert_point2 = '\0';
  sendto_one(source_p, "%s", buf);
}
