/* m_locops.c: Sends a message to all operators on the local server.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "send.h"
#include "s_user.h"
#include "s_conf.h"
#include "hash.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "cluster.h"
#include "s_serv.h"

static void m_locops(struct Client *, struct Client *, int, char **);
static void ms_locops(struct Client *, struct Client *, int, char **);

struct Message locops_msgtab = {
  "LOCOPS", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, ms_locops, m_ignore, m_locops}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&locops_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&locops_msgtab);
}
const char *_version = "1.41.2.2";
#endif

/* m_locops - write to *all* local opers currently online
 *  parv[0] = sender prefix
 *  parv[1] = message text
 */
static void m_locops(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (EmptyString(parv[1])) {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0],
      "LOCOPS");
    return;
  }
  sendto_wallops_flags(UMODE_LOCOPS, source_p, "Broadcast Message (Local "
    "Operators) - %s", parv[1]);
  if (dlink_list_length(&cluster_list) > 0)
    cluster_generic(source_p, "LOCOPS", CLUSTER_LOCOPS, CAP_CLUSTER, ":%s",
      parv[1]);
}

/* ms_locops - clustered LOCOPS
 *  parv[0] = sender prefix
 *  parv[1] = target server
 *  parv[2] = message
 */
static void ms_locops(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (parc != 3 || EmptyString(parv[2]))
    return;
  sendto_match_servs(source_p, parv[1], CAP_CLUSTER, NOCAPS, "LOCOPS %s :%s",
    parv[1], parv[2]);
  if (!match(parv[1], me.name))
    return;
  if (!IsPerson(source_p))
    return;
  if (find_cluster(source_p->user->server, CLUSTER_LOCOPS))
    sendto_wallops_flags(UMODE_LOCOPS, source_p, "Broadcast Message (Local "
      "Cluster Operators) - %s", parv[2]);
}
