/* m_lusers.c: Sends user statistics.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_serv.h"
#include "s_user.h"
#include "send.h"
#include "s_conf.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void m_lusers(struct Client *, struct Client *, int, char **);
static void ms_lusers(struct Client *, struct Client *, int, char **);

struct Message lusers_msgtab = {
  "LUSERS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_lusers, ms_lusers, m_ignore, ms_lusers}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&lusers_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&lusers_msgtab);
}
const char *_version = "1.37.2.2";
#endif

/* m_lusers - LUSERS message handler
 *  parv[0] = sender
 *  parv[1] = host/server mask.
 *  parv[2] = server to query
 *
 * ignores parv[1] completely and require parc > 3 to cause a force
 */
static void m_lusers(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  static time_t last_used = 0;
  if (parc > 2 && !ConfigServerHide.disable_remote) {
    if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime) {
      /* safe enough to give this on a local connect only */
      sendto_one(source_p, form_str(RPL_LOAD2HI),
        me.name, parv[0], "LUSERS");
      return;
    }
    else
      last_used = CurrentTime;
    if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2, parc, parv) !=
      HUNTED_ISME)
      return;
  }
  show_lusers(source_p);
}
/*
 * ms_lusers - LUSERS message handler for servers and opers
 *  parv[0] = sender
 *  parv[1] = host/server mask.
 *  parv[2] = server to query
 *
 * ignores parv[1] completely and require parc > 3 to cause a force
 */
static void ms_lusers(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (parc > 2) {
    if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2, parc, parv)
      != HUNTED_ISME) {
      return;
    }
  }
  if (IsClient(source_p))
    show_lusers(source_p);
}
