/*
 *  m_map.c: Sends an Undernet compatible map to a user.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *  Copyright (C) 2002      ircd-ratbox development team
 *  Copyright (C) 2003-2004 ircd-immortal development team
 *
 *  This file is licensed under the GNU General Public License, which is
 *  distributed as LICENSE.
 */

#include "stdinc.h"
#include "client.h"
#include "modules.h"
#include "handlers.h"
#include "numeric.h"
#include "send.h"
#include "s_conf.h"
#include "hook.h"

#define USER_COL 50 /* display | Users: %d at col 50 */

static void m_map(struct Client * client_p, struct Client * source_p,
   int parc, char *parv[]);
static void mo_map(struct Client * client_p, struct Client * source_p,
 int parc, char *parv[]);
static void dump_map(struct Client * client_p, struct Client * root,
  char *pbuf);
static void links_spy(struct Client * client_p, struct Client * source_p,
  const char *mask);

struct Message map_msgtab = {
  "MAP", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_map, m_ignore, m_ignore, mo_map}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&map_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&map_msgtab);
}
const char *_version = "1.28.2.2";
#endif

static char buf[BUFSIZE];
/* m_map
 *  parv[0] = sender prefix
 */
static void m_map(struct Client * client_p, struct Client * source_p,
   int parc, char *parv[])
{
  if (!ConfigFileEntry.map_oper_only && !ConfigServerHide.flatten_links) {
    mo_map(client_p, source_p, parc, parv);
    return;
  }
  m_not_oper(client_p, source_p, parc, parv);
  return;
}
/* mo_map
 *  parv[0] = sender prefix
 */
static void mo_map(struct Client * client_p, struct Client * source_p,
   int parc, char *parv[])
{
  links_spy(client_p, source_p, "*");
  dump_map(client_p, &me, buf);
  sendto_one(client_p, form_str(RPL_MAPEND), me.name, client_p->name);
}
/* dump_map - dumps server map recursively */
static void dump_map(struct Client * client_p, struct Client * root_p,
  char *pbuf)
{
  int cnt = 0, i = 0, len;
  struct Client *server_p;
  dlink_node *ptr;
  *pbuf = '\0';
  strncat(pbuf, root_p->name, BUFSIZE - ((size_t) pbuf - (size_t) buf));
  len = strlen(buf);
  buf[len] = ' ';
  if (len < USER_COL) {
    for (i = len + 1; i < USER_COL; i++) {
      buf[i] = '-';
    }
  }
  snprintf(buf + USER_COL, BUFSIZE - USER_COL, " %4lu (%4.1f%%)",
    dlink_list_length(&root_p->serv->users), 100 * (float)dlink_list_length(
    &root_p->serv->users) / (float)Count.total);
  sendto_one(client_p, form_str(RPL_MAP), me.name, client_p->name, buf);
  if (root_p->serv->servers.head != NULL) {
    cnt += dlink_list_length(&root_p->serv->servers);
    if (cnt) {
      if (pbuf > buf + 3) {
        pbuf[-2] = ' ';
        if (pbuf[-3] == '`')
          pbuf[-3] = ' ';
      }
    }
  }
  i = 1;
  DLINK_FOREACH(ptr, root_p->serv->servers.head) {
    server_p = (struct Client *) ptr->data;
    *pbuf = ' ';
    if (i < cnt)
      *(pbuf + 1) = '|';
    else
      *(pbuf + 1) = '`';
    *(pbuf + 2) = '-';
    *(pbuf + 3) = ' ';
    dump_map(client_p, server_p, pbuf + 4);
    i++;
  }
}
static void links_spy(struct Client * client_p, struct Client * source_p,
  const char *mask)
{
  struct hook_links_data data;
  data.client_p = client_p;
  data.source_p = source_p;
  data.mask = mask;
  hook_call_event(h_doing_links_id, &data);
}
