/* m_motd.c: Shows the current message of the day.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "client.h"
#include "tools.h"
#include "motd.h"
#include "ircd.h"
#include "send.h"
#include "numeric.h"
#include "handlers.h"
#include "hook.h"
#include "msg.h"
#include "s_serv.h"
#include "parse.h"
#include "modules.h"
#include "s_conf.h"

static void mr_motd(struct Client *, struct Client *, int, char **);
static void m_motd(struct Client *, struct Client *, int, char **);
static void mo_motd(struct Client *, struct Client *, int, char **);
static void mo_omotd(struct Client *, struct Client *, int, char **);
static void motd_spy(struct Client *);

struct Message motd_msgtab = {
  "MOTD", 0, 0, 0, 1, MFLG_SLOW, 0,
  {m_unregistered, m_motd, mo_motd, m_ignore, mo_motd}
};
struct Message omotd_msgtab = {
  "OMOTD", 0, 0, 0, 1, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, mo_omotd, m_ignore, mo_omotd}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&motd_msgtab);
  mod_add_cmd(&omotd_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&motd_msgtab);
  mod_del_cmd(&omotd_msgtab);
}
const char *_version = "1.44.2.4";
#endif

/* m_motd
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void m_motd(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  static time_t last_used = 0L;
  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime) {
    /* safe enough to give this on a local connect only */
    sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, parv[0], "MOTD");
    sendto_one(source_p, form_str(RPL_ENDOFMOTD), me.name, parv[0]);
    return;
  }
  else
    last_used = CurrentTime;

  /* This is safe enough to use during non hidden server mode */
  if (!ConfigServerHide.disable_remote && !ConfigServerHide.hide_servers) {
    if (hunt_server(client_p, source_p, ":%s MOTD :%s", 1, parc, parv) !=
        HUNTED_ISME)
      return;
  }
  hook_call_event(h_doing_motd_id, source_p);
  SendMessageFile(source_p, &ConfigFileEntry.motd);
}

/* mo_motd
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void mo_motd(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (!IsClient(source_p))
    return;
  if (hunt_server(client_p, source_p, ":%s MOTD :%s", 1, parc, parv) !=
      HUNTED_ISME)
    return;
  hook_call_event(h_doing_motd_id, source_p);
  SendMessageFile(source_p, &ConfigFileEntry.motd);
}

/* mo_omotd
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void mo_omotd(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (!IsClient(source_p))
    return;
  if (hunt_server(client_p, source_p, ":%s OMOTD :%s", 1, parc, parv) !=
      HUNTED_ISME)
    return;
  hook_call_event(h_doing_motd_id, source_p);
  SendMessageFile(source_p, &ConfigFileEntry.opermotd);
}
