/* m_operwall.c: Sends a message to all IRCOps.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "send.h"
#include "s_user.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void mo_operwall(struct Client *, struct Client *, int, char **);
static void ms_operwall(struct Client *, struct Client *, int, char **);

struct Message operwall_msgtab = {
  "OPERWALL", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, ms_operwall, m_ignore, mo_operwall}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&operwall_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&operwall_msgtab);
}
const char *_version = "1.46.2.4";
#endif

/* mo_operwall - write to *all* opers currently online
 *  parv[0] = sender prefix
 *  parv[1] = message text
 */
static void mo_operwall(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  char *message = parv[1];
  if (!IsOperOperwall(source_p)) {
    sendto_one(source_p, ":%s NOTICE %s :You do not have sufficient "
      "priviledges to send operator broadcast messages", me.name, parv[0]);
    return;
  }
  if (EmptyString(message)) {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
      me.name, parv[0], "OPERWALL");
    return;
  }
  sendto_wallops_flags(UMODE_OPERWALL, source_p, "Broadcast Message (All "
    "Operators) - %s", message);
  sendto_server(NULL, NULL, NOCAPS, NOCAPS, ":%s OPERWALL :%s",
    parv[0], message);
}

/* ms_operwall - write to *all* local opers currently online
 *  parv[0] = sender prefix
 *  parv[1] = message text
 */
static void ms_operwall(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  char *message = parv[1];
  if (EmptyString(message)) {
    if (MyClient(source_p))
      sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0],
        "OPERWALL");
    return;
  }
  sendto_wallops_flags(UMODE_OPERWALL, source_p, "Broadcast Message (All "
    "(All Operators) - %s", message);
  sendto_server(client_p, NULL, NOCAPS, NOCAPS, ":%s OPERWALL :%s", parv[0],
    message);
}
