/* m_opme.c: hacks operator status in opless channels
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "whowas.h"
#include "irc_string.h"
#include "hash.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void mo_opme(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[]);

struct Message opme_msgtab = {
  "OPME", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_opme}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&opme_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&opme_msgtab);
}
const char *_version = "1.38.2.7";
#endif

/* mo_opme
 *  parv[0] = sender prefix
 *  parv[1] = channel
 */
static void mo_opme(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  struct Channel *chptr;
  struct membership *msptr;
  dlink_node *ptr;

  if (!IsAdmin(source_p)) {
    sendto_one(source_p, ":%s NOTICE %s :You must be an administrator to "
      "perform that command.", me.name, parv[0]);
    return;
  }

  /* target channel existence */
  chptr = hash_find_channel(parv[1]);
  if (chptr == NULL) {
    sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL), me.name, parv[0],
      parv[1]);
    return;
  }

  /* source_p channel membership */
  msptr = find_channel_membership(chptr, source_p);
  if (msptr == NULL) {
    sendto_one(source_p, ":%s NOTICE %s :You are not a member of channel %s.",
      me.name, parv[0], parv[1]);
    return;
  }

  /* opless state of channel */
  DLINK_FOREACH(ptr, chptr->members.head) {
    msptr = ptr->data;
    if (is_chanop(msptr)) {
      sendto_one(source_p, ":%s NOTICE %s :Channel %s is not without channel "
        "operators.", me.name, parv[0], parv[1]);
      return;
    }
  }

  /* always send warnings for local channels */
  msptr->flags |= CHFL_CHANOP;
  sendto_wallops_flags(UMODE_WALLOP, &me, "%s is hacking operator status for "
    "channel %s", get_oper_name(source_p), parv[1]);
  ilog(L_NOTICE, "OPME %s by %s", parv[1], log_client_name(source_p));

  if (*chptr->chname != '&') {
    sendto_server(NULL, NULL, NOCAPS, NOCAPS, ":%s WALLOPS :%s is hacking "
      "operator status for channel %s", me.name, get_oper_name(source_p),
      parv[1]);
    sendto_server(NULL, chptr, NOCAPS, NOCAPS, ":%s PART %s", parv[0], parv[1]);
    sendto_server(NULL, chptr, NOCAPS, NOCAPS, ":%s SJOIN %ld %s + :@%s",
       me.name, (signed long)chptr->channelts, parv[1], parv[0]);
  }
  sendto_channel_local(ALL_MEMBERS, chptr, ":%s MODE %s +o %s", me.name,
    parv[1], parv[0]);
}
