/* m_pass.c: Used to send a password for a server or client{} block.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "irc_string.h"
#include "send.h"
#include "numeric.h"
#include "ircd.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void mr_pass(struct Client *, struct Client *, int, char **);

struct Message pass_msgtab = {
  "PASS", 0, 0, 2, 0, MFLG_SLOW | MFLG_UNREG, 0,
  {mr_pass, m_registered, m_ignore, m_ignore, m_registered}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&pass_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&pass_msgtab);
}
const char *_version = "1.36.2.3";
#endif
/* m_pass - Added Sat, 4 March 1989
 * mr_pass - PASS message handler
 *  parv[0] = sender prefix
 *  parv[1] = password
 *  parv[2] = optional extra version information
 */
static void mr_pass(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  const char *password = parv[1];
  if (EmptyString(password)) {
    sendto_one(client_p, form_str(ERR_NEEDMOREPARAMS), me.name,
      BadPtr(parv[0]) ? "*" : parv[0], "PASS");
    return;
  }
  strlcpy(client_p->localClient->passwd, password,
    sizeof(client_p->localClient->passwd));
  if (parc > 2) {
    /* check if there is a :TS */
    if (irccmp(parv[2], "TS") == 0 && client_p->tsinfo == 0)
      client_p->tsinfo = TS_DOESTS;
  }
}
