/* m_restart.c: Exits and re-runs ircd.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"
#include "irc_string.h"
#include "sprintf_irc.h"
#include "ircd.h"
#include "numeric.h"
#include "s_conf.h"
#include "restart.h"
#include "s_log.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void mo_restart(struct Client *, struct Client *, int, char **);

struct Message restart_msgtab = {
  "RESTART", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_restart}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&restart_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&restart_msgtab);
}
const char *_version = "1.36.2.2";
#endif

/* mo_restart */
static void mo_restart(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  char buf[BUFSIZE];
  dlink_node *ptr;
  struct Client *target_p;
  if (!MyClient(source_p) || !IsOper(source_p)) {
    sendto_one(source_p, form_str(ERR_NOPRIVILEGES), me.name, parv[0]);
    return;
  }
  if (!IsOperDie(source_p)) {
    sendto_one(source_p, ":%s NOTICE %s :You have insufficient permissions to "
      "restart the server.", me.name, parv[0]);
    return;
  }
  if (parc < 2) {
    sendto_one(source_p, ":%s NOTICE %s :You must specify the server name with "
      "the RESTART command as confirmation.", me.name, parv[0], me.name);
    return;
  }
  else {
    if (irccmp(parv[1], me.name)) {
      sendto_one(source_p, ":%s NOTICE %s :The server name you specified does "
        "not match %s, and you may not remotely restart servers.",
        me.name, parv[0], me.name);
      return;
    }
  }
  DLINK_FOREACH(ptr, lclient_list.head) {
    target_p = ptr->data;
    sendto_one(target_p, ":%s NOTICE %s :Server Restarting by request of %s",
      me.name, target_p->name, get_client_name(source_p, HIDE_IP, 1));
  }
  DLINK_FOREACH(ptr, serv_list.head) {
    target_p = ptr->data;
    sendto_one(target_p, ":%s ERROR :Server restarting by the request of %s",
      me.name, get_oper_name(source_p));
  }
  ircsprintf(buf, "Server restarted by %s", get_client_name(source_p, HIDE_IP,
    1));
  restart(buf);
}
