/* m_spy.c: for monitoring important user actions
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "modules.h"
#include "hook.h"
#include "client.h"
#include "ircd.h"
#include "send.h"
#include "common.h"

int show_admin(struct Client *);
int show_motd(struct Client *);
int show_stats(struct hook_stats_data *);
int show_trace(struct Client *);
int show_info(struct Client *);
int show_links(struct hook_links_data *);

#ifndef STATIC_MODULES
void _modinit(void)
{
#endif
  hook_add_hook("doing_admin", (hookfn) show_admin);
  hook_add_hook("doing_motd", (hookfn) show_motd);
  hook_add_hook("doing_stats", (hookfn) show_stats);
  hook_add_hook("doing_trace", (hookfn) show_trace);
  hook_add_hook("doing_info", (hookfn) show_info);
  hook_add_hook("doing_links", (hookfn) show_links);
#ifndef STATIC_MODULES
}
void _moddeinit(void)
{
  hook_del_hook("doing_admin", (hookfn) show_admin);
  hook_del_hook("doing_motd", (hookfn) show_motd);
  hook_del_hook("doing_stats", (hookfn) show_stats);
  hook_del_hook("doing_trace", (hookfn) show_trace);
  hook_del_hook("doing_info", (hookfn) show_info);
  hook_del_hook("doing_links", (hookfn) show_links);
}
const char *_version = "1.9.4.5";
#endif

int show_admin(struct Client * source_p)
{
  sendto_realops_flags(UMODE_SPY, L_ALL, "Administration Information requested "
    "by %s{%s}", get_client_name(source_p, HIDE_IP, YES),
    source_p->user->server);
  return NO;
}
int show_motd(struct Client * source_p)
{
  sendto_realops_flags(UMODE_SPY, L_ALL, "Message of the Day requested by "
    "%s{%s}", get_client_name(source_p, HIDE_IP, YES),
    source_p->user->server);
  return NO;
}
int show_stats(struct hook_stats_data * data)
{
  if (((data->statchar == 'L') || (data->statchar == 'l')) &&
    (data->name != NULL)) {
      sendto_realops_flags(UMODE_SPY, L_ALL, "Statistics (%c for %s) requested "
        "by %s{%s}", data->statchar, data->name, get_client_name(data->source_p,
        HIDE_IP, YES), data->source_p->user->server);
  } else {
    sendto_realops_flags(UMODE_SPY, L_ALL, "Statistics (%c) requested by "
      "%s{%s}", data->statchar, get_client_name(data->source_p, HIDE_IP, YES),
      data->source_p->user->server);
  }
  return NO;
}
int show_trace(struct Client * source_p)
{
  sendto_realops_flags(UMODE_SPY, L_ALL, "Trace requested by %s{%s}",
    get_client_name(source_p, HIDE_IP, YES), source_p->user->server);
  return NO;
}
int show_info(struct Client * source_p)
{
  sendto_realops_flags(UMODE_SPY, L_ALL, "Server Information requested by "
    "%s{%s}", get_client_name(source_p, HIDE_IP, YES), source_p->user->server);
  return NO;
}
int show_links(struct hook_links_data * data)
{
  sendto_realops_flags(UMODE_SPY, L_ALL, "Network Links List (Mask: %s) "
    "requested by %s", EmptyString(data->mask) ? "*" : data->mask,
    get_client_name(data->source_p, HIDE_IP, YES));
  return NO;
}
