/* m_time.c: Sends the current time on the server.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_misc.h"
#include "s_conf.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "packet.h"

static void m_time(struct Client *, struct Client *, int, char **);
static void mo_time(struct Client *, struct Client *, int, char **);

struct Message time_msgtab = {
  "TIME", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_time, mo_time, m_ignore, mo_time}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&time_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&time_msgtab);
}
const char *_version = "1.39.2.2";
#endif

/* m_time
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void m_time(struct Client * client_p, struct Client * source_p, int parc,
  char *parv[])
{
  /* this is not rate limited, so end the grace period */
  if (MyClient(source_p) && !IsFloodDone(source_p))
    flood_endgrace(source_p);
  /* This is safe enough to use during non hidden server mode */
  if (!ConfigServerHide.disable_remote) {
    if (hunt_server(client_p, source_p, ":%s TIME :%s", 1, parc, parv) !=
        HUNTED_ISME)
      return;
  }
  sendto_one(source_p, form_str(RPL_TIME), me.name, parv[0], me.name, date(0));
}

/* mo_time
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void mo_time(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (hunt_server(client_p, source_p, ":%s TIME :%s", 1, parc, parv) ==
      HUNTED_ISME)
    sendto_one(source_p, form_str(RPL_TIME), me.name, parv[0], me.name,
      date(0));
}
