/* m_user.c: Sends username information.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_user.h"
#include "send.h"
#include "s_conf.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

#define UFLAGS (FLAGS_INVISIBLE|FLAGS_WALLOP|FLAGS_SERVNOTICE)

static void mr_user(struct Client *, struct Client *, int, char **);

struct Message user_msgtab = {
  "USER", 0, 0, 5, 0, MFLG_SLOW, 0L,
  {mr_user, m_registered, m_ignore, m_ignore, m_registered}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&user_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&user_msgtab);
}
const char *_version = "1.36.2.2";
#endif

/* mr_user
 *  parv[0] = sender prefix
 *  parv[1] = username (login name, account)
 *  parv[2] = client host name (from other servers)
 *  parv[3] = server host name (from other servers)
 *  parv[4] = real name
 */
static void mr_user(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  char *p;
  if ((p = strchr(parv[1], '@')))
    *p = '\0';
  if (EmptyString(parv[4])) {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name,
      BadPtr(parv[0]) ? "*" : parv[0], "USER");
    return;
  }
  do_local_user(parv[0], client_p, source_p, parv[1], parv[2], parv[3],
    parv[4]);
}
