/* m_users.c: Gives some basic user statistics.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_serv.h"
#include "s_conf.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void m_users(struct Client *, struct Client *, int, char **);
static void mo_users(struct Client *, struct Client *, int, char **);

struct Message users_msgtab = {
  "USERS", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_users, mo_users, m_ignore, mo_users}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&users_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&users_msgtab);
}
const char *_version = "1.36.2.2";
#endif

/* m_users
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void m_users(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (!ConfigServerHide.disable_remote) {
    if (hunt_server(client_p, source_p, ":%s USERS :%s", 1, parc, parv) !=
      HUNTED_ISME)
      return;
  }
  sendto_one(source_p, form_str(RPL_LOCALUSERS), me.name, parv[0],
    ConfigServerHide.hide_servers ? Count.total : Count.local,
    ConfigServerHide.hide_servers ? Count.max_tot : Count.max_loc);
  sendto_one(source_p, form_str(RPL_GLOBALUSERS), me.name, parv[0],
    Count.total, Count.max_tot);
}
/* mo_users
 *  parv[0] = sender prefix
 *  parv[1] = servername
 */
static void mo_users(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (hunt_server(client_p, source_p, ":%s USERS :%s", 1, parc, parv) ==
    HUNTED_ISME) {
    if (!IsOper(source_p) && ConfigServerHide.hide_servers)
      sendto_one(source_p, form_str(RPL_LOCALUSERS), me.name, parv[0],
        Count.total, Count.max_tot);
    else
      sendto_one(source_p, form_str(RPL_LOCALUSERS), me.name, parv[0],
        Count.local, Count.max_loc);
    sendto_one(source_p, form_str(RPL_GLOBALUSERS), me.name, parv[0],
      Count.total, Count.max_tot);
  }
}
