/* m_wallops.c: Sends a message to all operators.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "send.h"
#include "s_user.h"
#include "s_conf.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void ms_wallops(struct Client *, struct Client *, int, char **);
static void mo_wallops(struct Client *, struct Client *, int, char **);

struct Message wallops_msgtab = {
  "WALLOPS", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, ms_wallops, m_ignore, mo_wallops}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&wallops_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&wallops_msgtab);
}
const char *_version = "1.45.2.5";
#endif

/* mo_wallops - write to all online opers
 *  parv[0] = sender prefix
 *  parv[1] = message text
 */
static void mo_wallops(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  char *message;
  message = parv[1];
  if (EmptyString(message)) {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0],
      "WALLOPS");
    return;
  }
  sendto_wallops_flags(UMODE_OPERWALL, source_p, "Broadcast Message (All "
    "Operators) - %s", message);
  sendto_server(NULL, NULL, NOCAPS, NOCAPS, ":%s OPERWALL :%s", parv[0],
    message);
}

/* ms_wallops - write to all online opers
 *  parv[0] = sender prefix
 *  parv[1] = message text
 */
static void ms_wallops(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  char *message;
  message = parv[1];
  if (EmptyString(message)) {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0],
      "WALLOPS");
    return;
  }
  if (IsClient(source_p))
    sendto_wallops_flags(UMODE_OPERWALL, source_p, "Broadcast Message (All "
      "Operators) - %s", message);
  else
    sendto_wallops_flags(UMODE_WALLOP, source_p, "%s", message);
  sendto_server(client_p, NULL, NOCAPS, NOCAPS, ":%s WALLOPS :%s", parv[0],
    message);
}
