/* m_whois.c: Shows who a user was.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "whowas.h"
#include "handlers.h"
#include "client.h"
#include "common.h"
#include "hash.h"
#include "irc_string.h"
#include "ircd.h"
#include "ircd_defs.h"
#include "numeric.h"
#include "s_serv.h"
#include "s_user.h"
#include "send.h"
#include "s_conf.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void m_whowas(struct Client *, struct Client *, int, char **);
static void mo_whowas(struct Client *, struct Client *, int, char **);
static int whowas_do(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[]);

struct Message whowas_msgtab = {
  "WHOWAS", 0, 0, 0, 0, MFLG_SLOW, 0L,
  {m_unregistered, m_whowas, m_error, m_ignore, mo_whowas}
};

#ifndef STATIC_MODULES
void _modinit(void)
{
  mod_add_cmd(&whowas_msgtab);
}
void _moddeinit(void)
{
  mod_del_cmd(&whowas_msgtab);
}
const char *_version = "1.41.2.3";
#endif

/* m_whowas
 *  parv[0] = sender prefix
 *  parv[1] = nickname queried
 */
static void m_whowas(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  static time_t last_used = 0L;
  if (parc < 2 || BadPtr(parv[1])) {
    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN), me.name, parv[0]);
    return;
  }
  if ((last_used + ConfigFileEntry.pace_wait_simple) > CurrentTime) {
    sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, parv[0], "WHOWAS");
    sendto_one(source_p, form_str(RPL_ENDOFWHOWAS), me.name, parv[0], parv[1]);
    return;
  }
  else
    last_used = CurrentTime;
  whowas_do(client_p, source_p, parc, parv);
}
static void mo_whowas(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  if (parc < 2) {
    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN), me.name, parv[0]);
    return;
  }
  whowas_do(client_p, source_p, parc, parv);
}
static int whowas_do(struct Client * client_p, struct Client * source_p,
  int parc, char *parv[])
{
  struct Whowas *temp;
  int cur = 0;
  int max = -1, found = 0;
  char *p, *nick;
  if (parc < 2 || BadPtr(parv[1])) {
    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN), me.name, parv[0]);
    return NO;
  }
  if (parc > 2)
    max = atoi(parv[2]);
  if (parc > 3)
    if (hunt_server(client_p, source_p, ":%s WHOWAS %s %s :%s", 3, parc, parv))
      return NO;

  if ((p = strchr(parv[1], ',')))
    *p = '\0';
  nick = parv[1];
  temp = WHOWASHASH[hash_whowas_name(nick)];
  found = 0;
  for (; temp; temp = temp->next) {
    if (!irccmp(nick, temp->name)) {
      sendto_one(source_p, form_str(RPL_WHOWASUSER), me.name, parv[0],
        temp->name, temp->username, temp->hostname, temp->realname);
      if (ConfigServerHide.hide_servers && !IsOper(source_p))
        sendto_one(source_p, form_str(RPL_WHOISSERVER), me.name, parv[0],
          temp->name, ServerInfo.network_name, myctime(temp->logoff));
      else
        sendto_one(source_p, form_str(RPL_WHOISSERVER), me.name, parv[0],
          temp->name, temp->servername, myctime(temp->logoff));
      cur++;
      found++;
    }
    if (max > 0 && cur >= max)
      break;
  }
  if (!found)
    sendto_one(source_p, form_str(ERR_WASNOSUCHNICK), me.name, parv[0], nick);
  sendto_one(source_p, form_str(RPL_ENDOFWHOWAS), me.name, parv[0], parv[1]);
  return NO;
}
