/* servlink.c: main servlink executable
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "setup.h"
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#ifdef HAVE_LIBCRYPTO
#include <openssl/evp.h>
#include <openssl/err.h>
#endif
#ifdef HAVE_LIBZ
#include <zlib.h>
#endif
#include "servlink.h"
#include "io.h"
#include "control.h"

static void usage(void);
struct slink_state in_state;
struct slink_state out_state;
struct fd_table fds[5] = {
  {0, read_ctrl, NULL},
  {0, NULL, NULL},
  {0, NULL, NULL},
  {0, NULL, NULL},
  {0, NULL, NULL}
};

static void usage(void)
{
  fprintf(stderr, "ircd-immortal server link v1.1\n\n");
  fprintf(stderr, "This is a subprogram of ircd-immortal and cannot be used "
    "alone.\n");
  exit(1);
}

int main(int argc, char *argv[])
{
  int max_fd = 0;
  int i, x;
#ifdef SERVLINK_DEBUG
  int GDBAttached = 0;
  while (!GDBAttached)
    sleep(1);
#endif
#ifdef HAVE_LIBCRYPTO
  /* load error strings */
  ERR_load_crypto_strings();
#endif
  /* Make sure we are running under ircd */
  if (argc != 6 || strcmp(argv[0], "-slink"))
    usage();
  for (i = 0; i < 5; i++) {
    fds[i].fd = atoi(argv[i + 1]);
    if (fds[i].fd < 0)
      exit(1);
  }
  for (i = 0; i < 5; i++) {
    if (fds[i].fd >= 255) {
      for (x = 0; x < 255; x++) {
        if (x != fds[0].fd && x != fds[1].fd && x != fds[2].fd && x !=
            fds[3].fd && x != fds[4].fd) {
          if (dup2(fds[i].fd, x) < 0)
            exit(1);
          fds[i].fd = x;
          break;
        }
      }
    }
    fcntl(fds[i].fd, F_SETFL, O_NONBLOCK);
    if (fds[i].fd > max_fd)
      max_fd = fds[i].fd;
  }
  /* enter io loop */
  io_loop(max_fd + 1);
  return 0; /* NOTREACHED */
}
