/* ircd_signal.c: system signalling functions.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "ircd_signal.h"
#include "ircd.h"
#include "restart.h"
#include "s_log.h"
#include "memory.h"
#include "s_bsd.h"

/* dummy_handler - if alarm is still being used, we need this */
static void dummy_handler(int sig)
{
  /* Empty */
}

/* sigterm_handler - exit the server */
static void sigterm_handler(int sig)
{
  ilog(L_CRIT, "Received signal SIGTERM, server terminating");
  exit(-1);
}

/* sighup_handler - reread the server configuration */
static void sighup_handler(int sig)
{
  dorehash = 1;
}

/* sigusr1_handler - reread the motd file */
static void sigusr1_handler(int sig)
{
  doremotd = 1;
}

static void sigchld_handler(int sig)
{
  int status;
  waitpid(-1, &status, WNOHANG);
}

/* sigint_handler - restart the server */
static void sigint_handler(int sig)
{
  static int restarting = 0;
  if (server_state.foreground) {
    ilog(L_WARN, "Received signal SIGINT, server terminating");
    exit(0);
  }
  else {
    ilog(L_WARN, "Received signal SIGINT, server restarting");
    if (restarting == 0) {
      restarting = 1;
      server_reboot();
    }
  }
}

/* setup_signals - initialize signal handlers for server */
void setup_signals()
{
  struct sigaction act;
  act.sa_flags = 0;
  act.sa_handler = SIG_IGN;
  sigemptyset(&act.sa_mask);
  sigaddset(&act.sa_mask, SIGPIPE);
  sigaddset(&act.sa_mask, SIGALRM);
  sigaddset(&act.sa_mask, SIGTRAP);
#ifdef SIGWINCH
  sigaddset(&act.sa_mask, SIGWINCH);
  sigaction(SIGWINCH, &act, 0);
#endif
  sigaction(SIGPIPE, &act, 0);
  sigaction(SIGTRAP, &act, 0);
  act.sa_handler = dummy_handler;
  sigaction(SIGALRM, &act, 0);
  act.sa_handler = sighup_handler;
  sigemptyset(&act.sa_mask);
  sigaddset(&act.sa_mask, SIGHUP);
  sigaction(SIGHUP, &act, 0);
  act.sa_handler = sigint_handler;
  sigaddset(&act.sa_mask, SIGINT);
  sigaction(SIGINT, &act, 0);
  act.sa_handler = sigterm_handler;
  sigaddset(&act.sa_mask, SIGTERM);
  sigaction(SIGTERM, &act, 0);
  act.sa_handler = sigusr1_handler;
  sigaddset(&act.sa_mask, SIGUSR1);
  sigaction(SIGUSR1, &act, 0);
  act.sa_handler = sigchld_handler;
  sigaddset(&act.sa_mask, SIGCHLD);
  sigaction(SIGCHLD, &act, 0);
}
