/*  m_error.c: Handles error messages from the other end.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "s_debug.h"
#include "msg.h"
#include "memory.h"
#include "s_log.h"
#include "s_conf.h"

struct Message error_msgtab = {
  "ERROR", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0,
  {m_error, m_ignore, ms_error, m_ignore, m_ignore}
};

/* At protocol level, ERROR has only one parameter, but it's called internally
 * from other functions -msa
 *  parv[0] = sender prefix
 *  parv[1+n] = parameters
 */
void m_error(struct Client * client_p, struct Client * source_p, int parc,
  char *parv[])
{
  const char *para;
  if (IsAnyServer(client_p)) {
    para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";
    ilog(L_ERROR, "ERROR from %s: %s", log_client_name(source_p),
      para);
    if (ConfigFileEntry.hide_error_messages < 2) {
      sendto_realops_flags(UMODE_ALL, L_ADMIN, "Received ERROR message from "
        "%s: %s", get_client_name(client_p, HIDE_IP, YES), para);
      if (!ConfigFileEntry.hide_error_messages)
        sendto_realops_flags(UMODE_ALL, L_OPER, "Received ERROR message from "
          "%s: %s", get_client_name(client_p, HIDE_IP, YES), para);
    }
  }
  exit_client(client_p, source_p, source_p, "ERROR");
}
void ms_error(struct Client * client_p, struct Client * source_p, int parc,
  char *parv[])
{
  const char *para;
#ifndef HIDE_SERVERS_IPS
  static int showip = HIDE_IP;
#else
  static int showip = MASK_IP;
#endif
  para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";
  ilog(L_ERROR, "ERROR from %s: %s", log_client_name(client_p), para);
  if (ConfigFileEntry.hide_error_messages == 2)
    return;
  if (client_p == source_p) {
    sendto_realops_flags(UMODE_ALL, L_ADMIN, "Received ERROR message from %s: "
      "%s", get_client_name(client_p, HIDE_IP, YES), para);
    if (!ConfigFileEntry.hide_error_messages)
      sendto_realops_flags(UMODE_ALL, L_OPER, "Received ERROR message from %s: "
        "%s", get_client_name(client_p, HIDE_IP, YES), para);
  }
  else {
    sendto_realops_flags(UMODE_ALL, L_ADMIN, "Received ERROR message from %s, "
      "via %s: %s", source_p->name, get_client_name(client_p, HIDE_IP, YES),
      para);
    if (!ConfigFileEntry.hide_error_messages)
      sendto_realops_flags(UMODE_ALL, L_OPER, "Received ERROR message from %s, "
        "via %s: %s", source_p->name, get_client_name(client_p, HIDE_IP, YES),
        para);
  }
}
