/* memory.c: Memory utilities.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#define WE_ARE_MEMORY_C
#include "stdinc.h"
#include "ircd_defs.h"
#include "ircd.h"
#include "irc_string.h"
#include "memory.h"
#include "client.h"
#include "send.h"
#include "tools.h"
#include "s_log.h"
#include "restart.h"

/* MyMalloc - allocate memory, call outofmemory on failure */
void * MyMalloc(size_t size)
{
  void *ret = calloc(1, size);
  if (ret == NULL)
    outofmemory();
  return ret;
}

/* MyRealloc - reallocate memory, call outofmemory on failure */
void * MyRealloc(void *x, size_t y)
{
  void *ret = realloc(x, y);
  if (ret == NULL)
    outofmemory();
  return ret;
}

void MyFree(void *x)
{
  if (x)
    free((x));
}

void _DupString(char **x, const char *y)
{
  (*x) = malloc(strlen(y) + 1);
  strcpy((*x), y);
}

/* outofmemory - report if there is a problem or abort */
void outofmemory()
{
  static int was_here = 0;
  if (was_here)
    abort();
  was_here = 1;
  ilog(L_CRIT, "Ran out of memory!");
  restart("Out of Memory");
}
