/* restart.c: Functions to allow the ircd to restart.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "tools.h"
#include "restart.h"
#include "common.h"
#include "fdlist.h"
#include "ircd.h"
#include "send.h"
#include "s_debug.h"
#include "s_log.h"
#include "client.h"
#include "memory.h"

/* external var */
extern char **myargv;
void restart(const char *mesg)
{
  static int was_here = NO; /* redundant due to restarting flag below */
  if (was_here)
    abort();
  was_here = YES;
  ilog(L_NOTICE, "Restarting server (%s), memory data limit: %ld", mesg,
    get_maxrss());
  server_reboot();
}

void server_reboot(void)
{
  int i;
  sendto_realops_flags(UMODE_ALL, L_ALL, "Restarting server");
  ilog(L_NOTICE, "Restarting server");
  /* comm_close() will be called, implying data flushing.
   * bah, for now, the program ain't coming back to here, so forcibly close
   * everything the "wrong" way for now, and just LEAVE
   */
  for (i = 0; i < MAXCONNECTIONS; ++i)
    close(i);
  unlink(pidFileName);
  execv(SPATH, myargv);
  exit(-1);
}
