/* s_gline.c: GLine global ban functions.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "client.h"
#include "common.h"
#include "config.h"
#include "irc_string.h"
#include "ircd.h"
#include "hostmask.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_misc.h"
#include "scache.h"
#include "send.h"
#include "msg.h"
#include "fileio.h"
#include "s_serv.h"
#include "s_gline.h"
#include "hash.h"
#include "event.h"
#include "memory.h"

dlink_list glines;
static void expire_glines(void);
static void expire_pending_glines(void);

/* add_gline - links given struct into gline llist
 * input - pointer to struct ConfItem
 */
void add_gline(struct ConfItem * aconf)
{
  dlinkAddTailAlloc(aconf, &glines);
  add_conf_by_address(aconf->host, CONF_GLINE, aconf->user, aconf);
}

/* find_is_glined
 * input  - hostname
 *        - username
 * output - pointer to struct ConfItem if user@host glined
 */
struct ConfItem * find_is_glined(const char *host, const char *user)
{
  dlink_node *gline_node;
  struct ConfItem *kill_ptr;
  DLINK_FOREACH(gline_node, glines.head) {
    kill_ptr = gline_node->data;
    if ((kill_ptr->user && (!user || match(kill_ptr->user, user))) &&
      (kill_ptr->host && (!host || match(kill_ptr->host, host)))) {
      return kill_ptr;
    }
  }
  return ((struct ConfItem *) NULL);
}

/* cleanup_glines - expire gline lists */
void cleanup_glines()
{
  expire_glines();
  expire_pending_glines();
}

/* expire_glines - expires old glines */
static void expire_glines()
{
  dlink_node *gline_node;
  dlink_node *next_node;
  struct ConfItem *kill_ptr;
  DLINK_FOREACH_SAFE(gline_node, next_node, glines.head) {
    kill_ptr = gline_node->data;
    /* these are in chronological order */
    if (kill_ptr->hold > CurrentTime)
      break;
    dlinkDestroy(gline_node, &glines);
    delete_one_address_conf(kill_ptr->host, kill_ptr);
  }
}

/* expire_pending_glines - expires pending glines */
static void expire_pending_glines()
{
  dlink_node *pending_node;
  dlink_node *next_node;
  struct gline_pending *glp_ptr;
  DLINK_FOREACH_SAFE(pending_node, next_node, pending_glines.head) {
    glp_ptr = pending_node->data;
    if (((glp_ptr->last_gline_time + GLINE_PENDING_EXPIRE) <= CurrentTime)
      || find_is_glined(glp_ptr->host, glp_ptr->user)) {
      MyFree(glp_ptr->reason1);
      MyFree(glp_ptr->reason2);
      MyFree(glp_ptr);
      dlinkDestroy(pending_node, &pending_glines);
    }
  }
}
