/* scache.c: Server names cache.
 *
 * Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 * Copyright (C) 1996-2002 Hybrid Development Team
 * Copyright (C) 2002      ircd-ratbox development team
 * Copyright (C) 2003-2004 ircd-immortal development team
 *
 * This file is licensed under the GNU General Public License, which is
 * distributed as LICENSE.
 */

#include "stdinc.h"
#include "client.h"
#include "common.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "scache.h"
#include "memory.h"

/* tokenize server names to save memory */
#define SCACHE_HASH_SIZE 257
typedef struct scache_entry {
  char name[HOSTLEN + 1];
  struct scache_entry *next;
} SCACHE;
static SCACHE *scache_hash[SCACHE_HASH_SIZE];

void clear_scache_hash_table(void)
{
  memset(scache_hash, 0, sizeof(scache_hash));
}
static int hash(const char *string)
{
  int hash_value;
  hash_value = 0;
  while (*string) {
    hash_value += ToLower(*string);
    /* I don't like auto increments inside macro calls -db */
    string++;
  }
  return hash_value % SCACHE_HASH_SIZE;
}

/* find_or_add
 * input        - server name
 * output       - pointer to string in server name token list
 * side effects - adds server to the list if it's not there
 * note, make sure the name is a verified, existing servername
 */
const char * find_or_add(const char *name)
{
  int hash_index;
  SCACHE *ptr;
  ptr = scache_hash[hash_index = hash(name)];
  for (; ptr; ptr = ptr->next) {
    if (!irccmp(ptr->name, name))
      return ptr->name;
  }
  ptr = (SCACHE *) MyMalloc(sizeof(SCACHE));
  s_assert(0 != ptr);
  strlcpy(ptr->name, name, sizeof(ptr->name));
  ptr->next = scache_hash[hash_index];
  scache_hash[hash_index] = ptr;
  return ptr->name;
}

/* count_scache
 * input - pointer to where to leave number of servers cached
 *       - pointer to where to leave total memory usage
 */
void count_scache(int *number_servers_cached, u_long * mem_servers_cached)
{
  SCACHE *scache_ptr;
  int i;
  *number_servers_cached = 0;
  *mem_servers_cached = 0;
  for (i = 0; i < SCACHE_HASH_SIZE; i++) {
    scache_ptr = scache_hash[i];
    while (scache_ptr) {
      *number_servers_cached = *number_servers_cached + 1;
      *mem_servers_cached = *mem_servers_cached + (strlen(scache_ptr->name) +
        sizeof(SCACHE *));
      scache_ptr = scache_ptr->next;
    }
  }
}
